package com.miya.draw.config.ahas;

import com.alibaba.csp.sentinel.adapter.servlet.callback.UrlBlockHandler;
import com.alibaba.csp.sentinel.slots.block.BlockException;
import com.alibaba.csp.sentinel.slots.block.authority.AuthorityException;
import com.alibaba.csp.sentinel.slots.block.degrade.DegradeException;
import com.alibaba.csp.sentinel.slots.block.flow.FlowException;
import com.alibaba.csp.sentinel.slots.block.flow.param.ParamFlowException;
import com.alibaba.csp.sentinel.slots.system.SystemBlockException;
import com.miya.draw.config.constant.ConStants;
import com.miya.draw.config.exception.BusinessException;
import com.miya.draw.enums.ErrorEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @ClassName SentinelBlockHandler
 * @Description 统一处理sentinel异常
 * @Author wangjun
 * @Date 2020/9/8 10:31
 * @Version 1.0
 **/
@Component
@Slf4j
public class SentinelBlockHandler implements UrlBlockHandler {

    @Override
    public void blocked(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, String s, BlockException e) throws IOException {
        log.error("触发抽奖限流限制，exception={},message={}", e, s);
        if (e instanceof FlowException) {
            throw new BusinessException(ErrorEnum.SENTINEL_FLOW_EXCEPTION, ConStants.DRAW_SENTINEL);
        } else if (e instanceof ParamFlowException) {
            throw new BusinessException(ErrorEnum.SENTINEL_PARAM_FLOW_EXCEPTION, ConStants.DRAW_SENTINEL);
        } else if (e instanceof SystemBlockException) {
            throw new BusinessException(ErrorEnum.SENTINEL_SYSTEM_EXCEPTION, ConStants.DRAW_SENTINEL);
        } else if (e instanceof DegradeException) {
            throw new BusinessException(ErrorEnum.SENTINEL_DEGRADE_EXCEPTION, ConStants.DRAW_SENTINEL);
        } else if (e instanceof AuthorityException) {
            throw new BusinessException(ErrorEnum.SENTINEL_AUTHORITY_EXCEPTION, ConStants.DRAW_SENTINEL);
        } else {
            throw new BusinessException(ErrorEnum.SENTINEL_UNKNOWN_EXCEPTION, ConStants.DRAW_SENTINEL);
        }
    }
}
