package com.miya.draw.config.exception;

import com.miya.draw.config.response.RestResult;
import com.miya.draw.config.response.RestResultUtil;
import com.miya.draw.enums.ErrorEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * Controller层异常捕捉器
 *
 * @author jack
 */
@ControllerAdvice
@Slf4j
public class MyControllerAdvice {

    /**
     * 全局异常捕捉处理
     */
    @ResponseBody
    @ExceptionHandler(value = BusinessException.class)
    public RestResult errorHandler(BusinessException ex) {
        log.error("自定义业务异常: {}", ex.getMessage());
        return RestResultUtil.createFailureResult(RestResult.create(), ex);
    }

    /**
     * 全局异常捕捉处理
     */
    @ResponseBody
    @ExceptionHandler(value = Exception.class)
    public RestResult errorHandler(Exception ex) {
        log.error("程序异常: {}", ex.getMessage());
        ex.printStackTrace();
        return RestResultUtil.createFailureResult(RestResult.create(), new BusinessException(ErrorEnum.SYSTEM_EXP, "系统火爆,请稍候再试!"));
    }

    /**
     * 全局异常捕捉处理
     */
    @ResponseBody
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public RestResult validErrorHandler(Exception ex) {
        log.error("参数效验异常: {}", ex.getMessage());
        MethodArgumentNotValidException me = (MethodArgumentNotValidException) ex;
        StringBuilder b = new StringBuilder();
        for (ObjectError error : me.getBindingResult().getAllErrors()) {
            b.append(error.getDefaultMessage()).append(",");
        }
        return RestResultUtil.createFailureResult(RestResult.create(), new BusinessException(ErrorEnum.PARAM_CHECK_EXP, b.toString()));
    }


}