/*
 * Dian.so Inc.
 * Copyright (c) 2004-2017 All Rights Reserved.
 */
package com.miya.draw.config.response;

import com.miya.draw.config.exception.BusinessException;
import com.miya.draw.enums.ErrorEnum;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;


/**
 * 结果处理类
 *
 * @author jack
 */
@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class RestResultUtil {

    /**
     * 生成失败结果
     *
     * @param exception
     * @param <T>
     * @return
     */
    public static <T> RestResult<T> createFailureResult(RestResult result, BusinessException exception) {
        result.setSuccess(false);
        ErrorEnum errorEnum = exception.getErrorEnum();
        result.setErrcode(errorEnum.getCode());
        result.setErrmsg(exception.getMessage());
        return result;
    }

    /**
     * 生成成功结果
     *
     * @param <T>
     * @return
     */
    public static <T> RestResult<T> createSuccessResult() {
        return createSuccessResult(RestResult.create(), null);
    }

    /**
     * 生成成功结果
     *
     * @param data
     * @param <T>
     * @return
     */
    public static <T> RestResult<T> createSuccessResult(T data) {
        return createSuccessResult(RestResult.create(), data);
    }

    /**
     * 生成成功结果
     *
     * @param data
     * @param <T>
     * @return
     */
    public static <T> RestResult<T> createSuccessResult(RestResult<T> result, T data) {
        result.setSuccess(true);
        result.setData(data);
        return result;
    }

    /**
     * 是否为成功的结果
     *
     * @param result 结果
     * @return true：成功的结果，false：失败的结果
     */
    public static boolean isSuccess(RestResult result) {
        return result != null && result.isSuccess();
    }

}
