package com.miya.draw.controller;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.Sets;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.connection.RedisConnection;
import org.springframework.data.redis.core.Cursor;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ScanOptions;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Objects;
import java.util.Set;

/**
 * @author jack
 */
@Slf4j
@Api(tags = "redis操作")
@RestController
@RequestMapping("/redis")
public class RedisController {

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;


    /**
     * 查询redis key
     *
     * @param payload {"key":"123"}
     * @return key
     */
    @PostMapping(value = "scanKeys")
    public String scanKey(@RequestBody String payload) {
        String key = JSON.parseObject(payload).getString("key");
        return JSON.toJSONString(scan("*" + key + "*"));
    }

    /**
     * 删除redis key
     *
     * @param payload {"key":"123"}
     * @return 删除
     */
    @PostMapping(value = "deleteKey")
    public String deleteKey(@RequestBody String payload) {
        String key = JSON.parseObject(payload).getString("key");
        Long delete = redisTemplate.delete(scan("*" + key + "*"));
        return JSON.toJSONString(delete);
    }


    /**
     * 自定义scan方法
     *
     * @param pattern 正则
     * @return Set<String>
     */
    private Set<String> scan(String pattern) {
        Set<String> keys = Sets.newHashSet();
        RedisConnection connection = null;
        try {
            if (Objects.isNull(redisTemplate.getConnectionFactory())) {
                return keys;
            }
            connection = redisTemplate.getConnectionFactory().getConnection();
            ScanOptions scanOptions = ScanOptions.scanOptions()
                    .match(pattern)
                    .count(100)
                    .build();
            Cursor<byte[]> cursor = connection.scan(scanOptions);
            while (cursor.hasNext()) {
                keys.add(new String(cursor.next()));
            }
        } catch (Exception e) {
            log.error("遍历key失败={}", e.getMessage());
        } finally {
            if (null != connection) {
                connection.close();
            }
        }

        return keys;
    }
}
