package com.miya.draw.manager;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.miya.draw.config.constant.ConStants;
import com.miya.draw.entity.Account;
import com.miya.draw.entity.AccountPrize;
import com.miya.draw.enums.YesOrNoEnum;
import com.miya.draw.model.SendCouponModel;
import com.miya.draw.service.IAccountPrizeService;
import com.miya.draw.service.IAccountService;
import com.miya.draw.util.RestTemplateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * 实现类
 *
 * @author jack
 */
@Component
@Slf4j
public class OtherManagerImpl implements IOtherManager {

    @Autowired
    private IAccountService accountService;

    @Autowired
    private IAccountPrizeService accountPrizeService;

    /**
     * H5授权公众号appid
     */
    @Value("${saas.appId}")
    private String saasAppId;

    @Value("${saas.url}")
    private String saasUrl;
    /**
     * ⼩程序appId
     */
    @Value("${saas.hhappId}")
    private String saasHHAppId;

    @Value("${saas.form}")
    private String sasaForm;

    private Map<String, String> getSignHeaders() {
        Map<String, String> headers = new HashMap<>(8);

        headers.put("HH-APP", saasHHAppId);
        headers.put("HH-CI", "saas-wechat-app");
        headers.put("HH-FROM", sasaForm);
        return headers;
    }

    @Override
    public void sendCoupon(String couponId, String phone, Long accountPrizeId) {
        try {
            SendCouponModel sendCouponModel = SendCouponModel.builder()
                    .appId(saasAppId)
                    .id(couponId)
                    .nickName(phone)
                    .build();
            Account account = accountService.getOne(new LambdaQueryWrapper<Account>().eq(Account::getPhone, phone));
            Map<String, String> headMap = getSignHeaders();
            headMap.put("Authorization", "bearer " + account.getToken());
            String result = RestTemplateUtil.postWithJsonWithRetry(saasUrl + "/coupon/auth/card/ticket/send/coupon", JSON.toJSONStringWithDateFormat(sendCouponModel
                    , "yyyy-MM-dd HH:mm:ss", SerializerFeature.DisableCircularReferenceDetect), headMap);
            log.info("发券，body : {}, result : {}", JSON.toJSONStringWithDateFormat(sendCouponModel
                    , "yyyy-MM-dd HH:mm:ss", SerializerFeature.DisableCircularReferenceDetect), JSON.toJSONString(result));
            JSONObject jsonObject = JSON.parseObject(result);
            String code = jsonObject.getString("code");
            AccountPrize accountPrize = new AccountPrize();
            accountPrize.setId(accountPrizeId);
            if (StringUtils.isNotBlank(code) && ConStants.SUCCESS_CODE.equalsIgnoreCase(code)) {
                accountPrize.setSendStatus(YesOrNoEnum.YES.getCode());
                accountPrize.setSendMessage("success");
            } else {
                accountPrize.setSendStatus(YesOrNoEnum.NO.getCode());
                accountPrize.setSendMessage(jsonObject.getString("msg"));
            }
            accountPrizeService.updateById(accountPrize);
        } catch (Exception e) {
            log.error("phone={},prize={},couponId={}代金券发券失败:{}", phone, accountPrizeId, couponId, e.getMessage());
        }
    }
}
