package com.miya.draw.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.miya.draw.config.constant.ConStants;
import com.miya.draw.config.exception.BusinessException;
import com.miya.draw.entity.AccountPrize;
import com.miya.draw.entity.ActivityGoods;
import com.miya.draw.entity.Prize;
import com.miya.draw.enums.ErrorEnum;
import com.miya.draw.enums.PrizeTypeEnum;
import com.miya.draw.mapper.AccountPrizeMapper;
import com.miya.draw.mapper.PrizeMapper;


import com.miya.draw.service.IAccountPrizeService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author jack
 * @since 2021-05-05
 */
@Service
public class AccountPrizeServiceImpl extends ServiceImpl<AccountPrizeMapper, AccountPrize> implements IAccountPrizeService {

    @Resource
    private AccountPrizeMapper accountPrizeMapper;



    @Override
    public Integer gainAlreadyDrawCount(String phone,Long activityId, LocalDateTime start, LocalDateTime end) {
        LambdaQueryWrapper<AccountPrize> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(AccountPrize::getAccountPhone, phone)
                .eq(AccountPrize::getActivityId,activityId)
                .between(AccountPrize::getCreateTime, start, end);
        return accountPrizeMapper.selectCount(wrapper);
    }

    @Override
    public List<AccountPrize> listMinePrize(String phone,Long activityId) {
        LambdaQueryWrapper<AccountPrize> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(AccountPrize::getAccountPhone, phone);
        wrapper.eq(AccountPrize::getActivityId,activityId);
        wrapper.ne(AccountPrize::getPrizeType, PrizeTypeEnum.NO_COUPON.getCode());
        wrapper.orderByDesc(AccountPrize::getId);
        return accountPrizeMapper.selectList(wrapper);
    }

    @Override
    public List<AccountPrize> listAllWinningInfo(Integer size,Long activityId) {
        LambdaQueryWrapper<AccountPrize> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(AccountPrize::getActivityId,activityId);
        wrapper.ne(AccountPrize::getPrizeType, PrizeTypeEnum.NO_COUPON.getCode());
        wrapper.orderByAsc(AccountPrize::getPrizeType);
        wrapper.last("limit " + size);
        return accountPrizeMapper.selectList(wrapper);
    }

    @Override
    public Integer countJoinActivityPeople(Long activityId) {
        QueryWrapper<AccountPrize> wrapper  = new QueryWrapper();
        wrapper.select("DISTINCT(account_phone)");
        wrapper.eq("activity_id",activityId);
        return accountPrizeMapper.selectCount(wrapper);
    }

    @Override
    public List<AccountPrize> gainAlreadyDrawCountByPrize(String phone, Long activityId, List<Long> prizeIds) {
        QueryWrapper<AccountPrize> wrapper = new QueryWrapper<>();
        wrapper.select("count(id) as count,prize_id ")
                .eq("activity_id",activityId)
                .eq("account_phone",phone)
                .in("prize_id", prizeIds)
                .last("group by prize_id");
        return accountPrizeMapper.selectList(wrapper);
    }

    /**
     * 获得获得商品的key
     *
     * @return
     */
    private String buildKey(String phone, Long activityId, Long prizeId) {
        //shizu:draw:activity:good
        return ConStants.SHIZU_DRAW_PREFIX + ConStants.SHIZU_ACTIVITY_PRIZE+":"+activityId+":"+prizeId+":"+phone;
    }
}
