package com.miya.draw.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.miya.draw.entity.SyncTrade;
import com.miya.draw.entity.Trade;
import com.miya.draw.mapper.TradeMapper;
import com.miya.draw.service.ITradeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author jack
 * @since 2021-05-10
 */
@Service
@DS("db2")
@Slf4j
// 如果有事务就挂起 因为是另一个数据源查询了
@Transactional(propagation = Propagation.NOT_SUPPORTED)
public class TradeServiceImpl extends ServiceImpl<TradeMapper, Trade> implements ITradeService {

    @Resource
    private TradeMapper tradeMapper;

    @Override
    public List<Trade> ListNeedSyncTrade(Long maxId, Integer maxSize) {
        LambdaQueryWrapper<Trade> wrapper = new LambdaQueryWrapper();
        wrapper.gt(Trade::getBillno, maxId);
//        wrapper.ge(Trade::getTrantime, LocalDateTimeUtil.beginOfDay(LocalDateTime.now()));
        wrapper.isNotNull(Trade::getIdentityid);
        wrapper.last("limit " + maxSize);
        return tradeMapper.selectList(wrapper);
    }

    @Override
    public boolean updateBatchByIdNoTrans(List<SyncTrade> syncTrades) {
        // 更新十足数据库出错了也就出错了
        List<Trade> batchTrade = new ArrayList<>();
        Trade trade;
        try {
            for (SyncTrade syncTrade : syncTrades) {
                trade = new Trade();
                trade.setBillno(Math.toIntExact(syncTrade.getBillNo()));
                trade.setStatus("Y");
                trade.setUpdatetime(LocalDateTime.now());
                batchTrade.add(trade);
            }
            return updateBatchById(batchTrade);
        } catch (Exception e) {
            log.error("回调更新十足数据库状态出错:{} list={}", e.getMessage(), JSON.toJSON(syncTrades));
        }
        return false;
    }
}
