package com.miya.draw.xxljob;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.miya.draw.entity.AccountPrize;
import com.miya.draw.entity.Winner;
import com.miya.draw.enums.PrizeTypeEnum;
import com.miya.draw.enums.YesOrNoEnum;
import com.miya.draw.model.SyncTradeConfigModel;
import com.miya.draw.service.IAccountPrizeService;
import com.miya.draw.service.IWinnerService;
import com.miya.draw.util.OrikaUtil;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.IJobHandler;
import com.xxl.job.core.handler.annotation.JobHandler;
import com.xxl.job.core.log.XxlJobLogger;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 十足抽奖结果同步
 *
 * @author jack
 */
@Slf4j
@JobHandler("drawResultSyncJob")
@Component
public class DrawResultSyncJob extends IJobHandler {

    static {
        OrikaUtil.classMap(AccountPrize.class, Winner.class)
                .field("id", "syncId")
                .byDefault().register();
    }

    /**
     * 一次最多取多少数据
     */
    private Integer maxSize = 500;

    @Autowired
    private IAccountPrizeService accountPrizeService;

    @Autowired
    private IWinnerService winnerService;

    @Override
    public ReturnT<String> execute(String s) throws Exception {
        XxlJobLogger.log("向十足同步开始:{}" + LocalDateTime.now().toString());
        log.info("向十足同步开始:{}" + LocalDateTime.now().toString());
        if (StringUtils.isNotBlank(s)) {
            SyncTradeConfigModel model = JSON.parseObject(s, SyncTradeConfigModel.class);
            maxSize = model.getMaxSize();
        }
        LambdaQueryWrapper<AccountPrize> wrapper = new LambdaQueryWrapper();
        wrapper.ne(AccountPrize::getPrizeType, PrizeTypeEnum.NO_COUPON.getCode());
        wrapper.eq(AccountPrize::getSyncStatus, YesOrNoEnum.NO);
        wrapper.orderByDesc(AccountPrize::getId);
        wrapper.last("limit " + maxSize);
        List<AccountPrize> accountPrizeList = accountPrizeService.list(wrapper);
        if (CollectionUtils.isNotEmpty(accountPrizeList)) {
            XxlJobLogger.log("同步内容:{}", accountPrizeList.stream().map(AccountPrize::getId).collect(Collectors.toList()));
            log.info("同步内容:{}", accountPrizeList.stream().map(AccountPrize::getId).collect(Collectors.toList()));
            List<Winner> winnerList = OrikaUtil.convertList(accountPrizeList, Winner.class);
            winnerService.saveOrUpdateBatchByDulpKey(winnerList);

            accountPrizeList.forEach(
                    accountPrize -> accountPrize.setSyncStatus(YesOrNoEnum.YES.getCode())
            );
            accountPrizeService.updateBatchById(accountPrizeList);
        }

        XxlJobLogger.log("向十足同步结束:" + LocalDateTime.now().toString());
        log.info("向十足同步结束:" + LocalDateTime.now().toString());
        return ReturnT.SUCCESS;
    }
}
